@@ -59,6 +59,16 @@ static void memcpy7(char *dest, char const *src, int count)
 /*}}}*/
 
 /* file name conversions */ 
+/* cpmIsFileChar      -- is character allowed in a name?         */ /*{{{*/
+int cpmIsFileChar(char c, int type)
+{
+  if (c&0x80) return 0;
+  if (type == CPMFS_DR3)
+    return (c>' ' && c!='<' && c!='>' && c!='.' && c!=',' && c!=';' && c!=':' && c!='=' && c!='?' && c!='*' && c!='[' && c!=']' && c!='|' && !islower(c));
+  else
+    return (c>' ' && c!='<' && c!='>' && c!='.'           && c!=';' && c!=':' && c!='=' && c!='?' && c!='*' && c!= '_' && !islower(c));
+}
+/*}}}*/
 /* splitFilename      -- split file name into name and extension */ /*{{{*/
 static int splitFilename(char const *fullname, int type, char *name, char *ext, int *user) 
 {
@@ -82,27 +92,40 @@ static int splitFilename(char const *fullname, int type, char *name, char *ext,
     boo="illegal CP/M filename";
     return -1;
   }
-  for (i=0; i<8 && fullname[i] && fullname[i]!='.'; ++i) if (!ISFILECHAR(i,fullname[i]))
+  for (i=0; i<8 && fullname[i] && fullname[i]!='.'; ++i)
   {
+    if (!cpmIsFileChar(toupper(fullname[i]),type))
+    {
+      boo="illegal CP/M filename";
+      return -1;
+    }
+    else name[i]=toupper(fullname[i]);
+  }
+  if (i==0)
+  {
+    /* no filename after user or extension without filename */
     boo="illegal CP/M filename";
     return -1;
   }
-  else name[i]=toupper(fullname[i]);
   if (fullname[i]=='.')
   {
     ++i;
-    for (j=0; j<3 && fullname[i]; ++i,++j) if (!ISFILECHAR(1,fullname[i]))
+    for (j=0; j<3 && fullname[i]; ++i,++j)
     {
-      boo="illegal CP/M filename";
-      return -1;
-    }
-    else ext[j]=toupper(fullname[i]);
-    if (i==1 && j==0)
-    {
-      boo="illegal CP/M filename";
-      return -1;
+      if (!cpmIsFileChar(toupper(fullname[i]),type))
+      {
+        boo="illegal CP/M filename";
+        return -1;
+      }
+      else ext[j]=toupper(fullname[i]);
     }
   }
+  if (fullname[i])
+  {
+    /* too long name */
+    boo="illegal CP/M filename";
+    return -1;
+  }
   return 0;
 }
 /*}}}*/
@@ -981,6 +1004,7 @@ static int amsReadSuper(struct cpmSuperBlock *d, char const *format)
   d->skew      = 1; /* Amstrads skew at the controller level */
   d->skewtab   = (int*)0;
   d->boottrk   = boot_spec[5];
+  d->bootsec   = -1;
   d->offset    = 0;
   d->size      = (d->secLength*d->sectrk*(d->tracks-d->boottrk))/d->blksiz;
   d->extents   = ((d->size>256 ? 8 : 16)*d->blksiz)/16384;
