@@ -72,6 +72,7 @@ const char *Device_open(struct Device *this, const char *filename, int mode, con
 const char *Device_setGeometry(struct Device *this, int secLength, int sectrk, int tracks, off_t offset, const char *libdskGeometry)
 {
   char *boo;
+  int probeOk;
 
   this->secLength=secLength;
   this->sectrk=sectrk;
@@ -85,11 +86,22 @@ const char *Device_setGeometry(struct Device *this, int secLength, int sectrk, i
     return lookupFormat(&this->geom, libdskGeometry);
   }
   
+  /* Did the autoprobe guess right about the number of sectors & cylinders? */
+  if (this->geom.dg_cylinders * this->geom.dg_heads == tracks)
+  {
+    probeOk = 1;
+  }
+  else
+  {
+    /* If not, reset to a minimal geometry (to undo any randomness from
+     * a failed autoprobe), and guess some parameters */
+    probeOk = 0;
+    dg_stdformat(&this->geom, FMT_180K, NULL, NULL);
+  }
   this->geom.dg_secsize   = secLength;
   this->geom.dg_sectors   = sectrk;
-  /* Did the autoprobe guess right about the number of sectors & cylinders? */
-  if (this->geom.dg_cylinders * this->geom.dg_heads == tracks) return NULL;
-  /* Otherwise we guess: <= 43 tracks: single-sided. Else double. This
+  if (probeOk) return NULL;
+  /* We guess: <= 43 tracks: single-sided. Else double. This
    * fails for 80-track single-sided if there are any such beasts */
   if (tracks <= 43) 
   {
